use std::collections::HashMap;

/// Full `Parser` method implementation
struct FlagRule<'a> {
    arg: &'a str,
}

struct ActionRule<'a> {
    arg: &'a str,
    action: Box<Fn()>,
}

pub struct Parser<'a> {
    args: &'a [String],
    flag_rules: Vec<FlagRule<'a>>,
    action_rules: Vec<ActionRule<'a>>,
    flag_cache: HashMap<&'a str, &'a str>,
    action_cache: Vec<&'a str>,
}

impl<'a> Parser<'a> {
    // Constructor
    pub fn new(args: &'a [String]) -> Self {
        let flag_rules: Vec<FlagRule> = Vec::new();
        let action_rules: Vec<ActionRule> = Vec::new();
        Parser {
            args,
            flag_rules,
            action_rules,
            flag_cache: HashMap::new(),
            action_cache: Vec::new(),
        }
    }
    // Action / Flags
    pub fn add_flag(&mut self, arg: &'a str) {
        let flag_rule = FlagRule { arg };
        self.flag_rules.push(flag_rule);
    }
    pub fn add_action(&mut self, arg: &'a str, action: Box<Fn()>) {
        let action_rule = ActionRule { arg, action };
        self.action_rules.push(action_rule);
    }
    pub fn collect_flags(&mut self) -> HashMap<&str, &str> {
        let mut map: HashMap<&str, &str> = HashMap::new();
        let mut c = 0;
        for arg in self.args {
            for flag in &self.flag_rules {
                if *arg == flag.arg {
                    map.insert(&*arg, &self.args[c + 1]);
                }
            }
            c += 1;
        }
        self.flag_cache = map.clone();
        map
    }
    pub fn execute_actions(&mut self) -> Vec<&str> {
        let mut matches: Vec<&str> = Vec::new();
        for arg in self.args {
            for action in &self.action_rules {
                if *arg == action.arg {
                    matches.push(action.arg);
                    (action.action)();
                }
            }
        }
        self.action_cache = matches.clone();
        matches
    }
    // Arg Getters
    pub fn get_val(&self, val: &str) -> Option<&str> {
        if self.flag_cache.is_empty() {
            let mut c = 0;
            for arg in self.args {
                if arg == val {
                    if c == self.args.len() {
                        return None;
                    }
                    return Some(&self.args[c + 1]);
                }
                c += 1;
            }
        }

        match self.flag_cache.get(val) {
            Some(s) => Some(s),
            None => None,
        }
    }
    pub fn get_bool(&self, val: &str) -> bool {
        if self.action_cache.is_empty() {
            for arg in self.args {
                if arg == val {
                    return true;
                }
            }
        }

        if self.action_cache.contains(&val) {
            return true;
        }
        false
    }
}

#[cfg(test)]
mod tests;
