use std::collections::HashMap;

#[test]
pub fn collect_flags() {
    let args: Vec<String> = vec![
        "--test".to_string(),
        "-open".to_string(),
        "test.txt".to_string(),
    ];
    let mut parser = super::Parser::new(&args);
    parser.add_action("--test", Box::new(|| println!("this is a test")));
    parser.add_flag("-open");
    // Action should do nothing in this case

    let mut map: HashMap<&str, &str> = HashMap::new();
    map.insert("-open", "test.txt");
    assert_eq!(parser.collect_flags(), map);
}

#[test]
pub fn collect_actions() {
    let args: Vec<String> = vec!["--test".to_string(), "--open".to_string()];
    let mut parser = super::Parser::new(&args);
    parser.add_action("--test", Box::new(|| println!("this is a test")));
    parser.add_action("--open", Box::new(|| println!("Ignore")));
    let actions = parser.execute_actions();
    assert_eq!(actions, vec!["--test", "--open"]);
}

#[test]
pub fn collect_actions_edge_case_01() {
    let args: Vec<String> = vec!["--test".to_string()];
    let mut parser = super::Parser::new(&args);
    parser.add_action("--open", Box::new(|| println!("Ignore")));
    let actions = parser.execute_actions();
    assert_eq!(actions, Vec::<&str>::new());
}

#[test]
pub fn collect_actions_edge_case_02() {
    let args: Vec<String> = vec!["--test".to_string(), "--open".to_string()];
    let mut parser = super::Parser::new(&args);
    parser.add_action("--test", Box::new(|| println!("Ignore")));
    let actions = parser.execute_actions();
    assert_eq!(actions, vec!["--test"]);
}

#[test]
pub fn get_bool() {
    let args: Vec<String> = vec!["--test".to_string()];
    let parser = super::Parser::new(&args);
    let b = parser.get_bool("--test");
    assert_eq!(true, b);
}

#[test]
pub fn get_bool_2() {
    let args: Vec<String> = vec!["--not-test".to_string()];
    let parser = super::Parser::new(&args);
    let b = parser.get_bool("--test");
    assert_eq!(false, b);
}

#[test]
pub fn get_val() {
    let args: Vec<String> = vec!["-p".to_string(), "test.txt".to_string()];
    let parser = super::Parser::new(&args);
    let v = parser.get_val("-p");
    assert_eq!(v, Some("test.txt"));
}

#[test]
pub fn get_val_2() {
    let args: Vec<String> = vec!["-notp".to_string(), "test.txt".to_string()];
    let parser = super::Parser::new(&args);
    let v = parser.get_val("-p");
    assert_eq!(v, None);
}